﻿using BPASmartClient.Helper;
using BPASmartClient.Modbus;
using DosingSystem.ViewModel;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Net.NetworkInformation;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace DosingSystem.Model
{
    public class DeviceInquire
    {

        private volatile static DeviceInquire _Instance;
        public static DeviceInquire GetInstance => _Instance ?? (_Instance = new DeviceInquire());
        private DeviceInquire() { }

        string IPSegment = "192.168.1.";
        string NameAddress = "VW5000";
        int count = 0;
        private static readonly object _lock = new object();
        ConcurrentDictionary<string, ModbusTcp> modbusTcps = new ConcurrentDictionary<string, ModbusTcp>();
        List<string> InvalidIP = new List<string>();

        public void Init()
        {
            IpAddressLines();
            ThreadManage.GetInstance().StartLong(new Action(() =>
            {
                if (count >= 255) IpAddressLines();
                Thread.Sleep(5000);
            }), "配料机设备上线监听", true);
        }

        public ModbusTcp GetModbusTcp(string ip)
        {
            if (modbusTcps.ContainsKey(ip)) return modbusTcps[ip];
            else return default;
        }

        private void IpAddressLines()
        {
            for (int i = 1; i <= 255; i++)
            {
                if (!InvalidIP.Contains($"{IPSegment}{i}"))
                {
                    Ping myPing;
                    myPing = new Ping();
                    myPing.PingCompleted += new PingCompletedEventHandler(_myPing_PingCompleted);
                    string pingIP = $"{IPSegment}{i}";
                    myPing.SendAsync(pingIP, 1000, null);
                }
            }
            count = 0;
        }

        private void CompleteCount()
        {
            lock (_lock)
                count++;
        }

        private void _myPing_PingCompleted(object sender, PingCompletedEventArgs e)
        {
            if (e.Reply.Status == IPStatus.Success)
            {
                string ip = e.Reply.Address.ToString();
                if (!modbusTcps.ContainsKey(ip))
                {
                    modbusTcps.TryAdd(ip, new ModbusTcp());
                    modbusTcps[ip].IsReconnect = false;
                    Task.Run(new Action(() => { modbusTcps[ip].ModbusTcpConnect(ip); }));

                    modbusTcps[ip].ConnectOk = new Action(() =>
                    {
                        string DeviceName = modbusTcps[ip].GetString(NameAddress, 10);
                        if (DeviceName.Length > 0)
                        {
                            App.Current.Dispatcher.Invoke(new Action(() =>
                            {
                                DeviceListViewModel.devices.Add(new Devices()
                                {
                                    DeviceName = DeviceName,
                                    IpAddress = ip
                                });
                            }));
                        }
                        else
                        {
                            if (!InvalidIP.Contains(ip)) InvalidIP.Add(ip);
                        }
                    });

                    modbusTcps[ip].ConnectFail = new Action(() =>
                    {
                        if (!InvalidIP.Contains(ip)) InvalidIP.Add(ip);
                    });

                    modbusTcps[ip].Disconnect = new Action(() =>
                    {
                        if (InvalidIP.Contains(ip)) InvalidIP.Remove(ip);
                        var res = DeviceListViewModel.devices.FirstOrDefault(P => P.IpAddress == ip);
                        if (res != null && DeviceListViewModel.devices.Contains(res))
                            App.Current.Dispatcher.Invoke(new Action(() => { DeviceListViewModel.devices.Remove(res); }));
                    });
                }
            }
            CompleteCount();
        }


    }
}
